# include <string.h>
# include <time.h>

# include "parts/m55800/eb55.h"
# include "parts/m55800/lib_m55800.h"
# include "periph/stdc/std_c.h"
# include "drivers/wait/wait.h"

# define ADBG	1
# define SERIAL_COUNTER	0x8

/* * * * * * * * * * * AKV-ACTION COMMANDS * * * * * * * * * * */
unsigned int AKV_SERIAL_NUM;

unsigned int LOCK 		=	0x11111111; 
unsigned int UNLOCK		=	0x11112222;

unsigned int ENGINE_ON		=	0x22222222;
unsigned int ENGINE_OFF 	= 	0x22223333;

unsigned int PANIC		=	0x33333333;
unsigned int UNLOCK_TRUNK	=	0x44444444;

unsigned int ENROLL_PASSED	=	0x55555555;
unsigned int ENROLL_MASTER_PASSED =	0x66666666;
unsigned int BIOMETRIC_FAILED	=	0x00000000;

unsigned int TIMEOUT		= 	0x00000000;

/* * * * * * * * * * * AKV-ACTION COMMANDS * * * * * * * * * * */
extern void wake_up_handler (void);

unsigned int getAKVcommand(int id, WaitDesc wait_desc)
{
    int buttonSeq[]={0,0,0,0};
    int buttons=0;
    int buttonPressed=0;

    int count = 1;
    time_t first;
    time(&first);

    //detect button presses
    for(;;)
    {
	u_int status = at91_pio_read (&PIOB_DESC) ;
	u_int astatus = at91_pio_read (&PIOA_DESC) ;

	//breakout conditions
	/****************************************/
	//double button press of the same button = double click function
	if (buttons==2 && buttonSeq[0]==buttonSeq[1])
	    break;

	//reached the maximum button presses we need
	if (buttons==4)
	    break;

	//timeout condition
	if (count > 250)//difftime(time(NULL), first)>3)
	{
	    //if one button is pressed, we are ok. consider this as a single button press
	    if (buttons==1)
		break;
	
	    //this is the case where no buttons are pressed, three buttons pressed, two different buttons pressed etc.
	    return TIMEOUT;
	}
	/****************************************/
	
	//check which button was pressed
	if ((status & (1<<PIO_SW1)) == 0 )
	{
	    at91_wait_open ( &wait_desc ) ;
	    buttonPressed=1;	    
	}
	else if ((astatus & (1<<PIO_SW2)) == 0 )
	{
	    at91_wait_open ( &wait_desc ) ;
	    buttonPressed=2;
	}
	else if ((status & (1<<PIO_SW3)) == 0 )
	{
	    at91_wait_open ( &wait_desc ) ;
	    buttonPressed=3;
	}
	else if ((status & (1<<PIO_SW4)) == 0 )
	{
	    buttonPressed=4;
	    at91_wait_open ( &wait_desc ) ;
	}

	if (buttonPressed)
	{
	    
	    //turn on led to indicate we got the button press
	    at91_pio_write (&PIOB_DESC, LED7, LED_ON );
	    wait_desc.period = 200000 ;
	    at91_wait_open ( &wait_desc ) ;
	    at91_pio_write (&PIOB_DESC, LED7, LED_OFF );

	    //store button in sequence
	    buttonSeq[buttons]=buttonPressed;
	    //update number of buttons we got so far
	    buttons++;
	    buttonPressed=0;
	}
	wait_desc.period = 10000 ;
	at91_wait_open ( &wait_desc ) ;
	count++;
	wait_desc.period = 200000 ;
    }

#if ADBG
    printf("button1=%d, button2:%d, button3:%d, button4:%d\n", buttonSeq[0], buttonSeq[1], buttonSeq[2], buttonSeq[3]);
#endif
    
    //check which buttons are pressed here:
    //check if it was a single press
    if (buttons==1)
    {
	switch(buttonSeq[0])
	{
	    case 1:
		return LOCK;
	    case 2:
		return ENGINE_ON;
	    case 3:
		return PANIC;
	    case 4:
		return UNLOCK_TRUNK;
	    default:
		return TIMEOUT;	
	}
    }

    //check if it was a double press
    //we KNOW that the two button pressed are THE SAME or else we would have gotten a timeout
    if (buttons==2)
    {
	switch(buttonSeq[0])
	{
	    case 1:
		return UNLOCK;
	    case 2:
		return ENGINE_OFF;
	    default:
		return TIMEOUT;	
	}
    }

    //check if it was 4 buttons pressed and if it was the enrollment sequence
    if (buttons==4)
    {
	int enrollmentStatus=0;
	
	if (id==1 && buttonSeq[0]==4 && buttonSeq[1]==3 && buttonSeq[2]==2 && buttonSeq[3]==1)
	{
	    //to fix- create function to ask biometric how much memory is used
	    if (memoryUsed()<0x77) //0x77 is 119 in hex
  	    {
  		enrollmentStatus=enrollFingerprint(memoryUsed()+1);
		if (enrollmentStatus==1)
		{
		    return ENROLL_PASSED;
		}
		else
		    return BIOMETRIC_FAILED;
  	    }
  	    else
		//memory full
  		return BIOMETRIC_FAILED;
	}
	/*sequence for deleting all fingerprints*/
	else if (id==1 && buttonSeq[0]==3 && buttonSeq[1]==4 && buttonSeq[2]==3 && buttonSeq[3]==4)
	{
	    int enrollmentStatus=0;
	    //delete all fingerprints
	    deleteAll();

	    //blink LED8 continuously

	    enrollmentStatus=enrollFingerprint(memoryUsed()+1);

	    while (enrollmentStatus!=1)
	    {
		enrollmentStatus=enrollFingerprint(memoryUsed()+1);
	    }
	    
	    return ENROLL_MASTER_PASSED;
	    
	}
    }

    return 0;
}
