#ifdef SEMIHOSTING
#include <stdio.h>
#endif

#include <string.h>
#include    "periph/stdc/std_c.h"
#include    "biometric.h"

//* Include files for EB55
#include "parts/m55800/eb55.h"
#include "parts/m55800/lib_m55800.h"

#include    "periph/usart/usart.h"
#define DBG 1

int receivedIndex=20;

int verifyFingerprint()
{
    int same;

    //check if there was a match or not
    same=checkStatus(noMatch, SIZE_OF(noMatch));
    
    if (same)
	//means status=noMatch therefore, no match was found
	return -1;
    else
	//means match was found
	return bioReceived[11]+1; //index that holds fingerprint id	
}

int checkStatus(u_int status[], int size)
{
    int i;
    int same=1; //indicates if received data is the same as the status or not..
    //if 1, status is the same; if 0, means different
    
    //check byte by byte to see if the statuses are the same
    for (i=0; i<size; i++)
    {
	if (status[i]!=bioReceived[i])
	{
	    same=0;
	    break;
	}
    }

    if (same==1)
	//means status is the same
	return 1;
    else
	//means different
	return 0;
}

void sendCommand(u_int command[], int size)
{
    int i;
    
    for (i=0; i<size; i++)
    {
	while(1)
	{
	    //Check usart status and get transmit status to see if its ready
	    if (at91_usart_get_status(&USART1_DESC) & US_TXRDY)
	    {
		//* Write character to usart
		at91_usart_write(&USART1_DESC,command[i]);
#if DBG		
		//printf("%x ", command[i]);
#endif
		break;
	    }
	}
    }
}


void initializeBioReceived()
{
    int i;
    for (i=0; i<SIZE_OF(bioReceived); i++)
    {
	bioReceived[i]=0x00;
    }
}

void getReply()
{
    int i;
    u_int packetLengthReceived=0;
    
    receivedIndex=20;

    //make sure the receive buffer is empty
    initializeBioReceived();
    
    //receive reply from command
    for (i=0; i<receivedIndex; i++)
    {
	if (i==9)
	{
	    //check the length of the packet and adjust the upper condition (in for loop) accordingly
	    packetLengthReceived=(bioReceived[7]<<8) + bioReceived[8];
	    receivedIndex=i+packetLengthReceived;
	}

	while(1)
	{
	    //check if USART is ready and if so, receive a byte
	    if (at91_usart_get_status(&USART1_DESC) & US_RXRDY)
	    {
		at91_usart_read(&USART1_DESC, &bioReceived[i]);
		break;
	    }		    
	}
    }
}

int deleteAll()
{
    initializeBioReceived();
    //delete all fingerprints
    sendCommand(deleteCommand, SIZE_OF(deleteCommand));
    getReply();

    //checkStatus will return 1 if status is OK, -1 if not ok
    return checkStatus(statusOK, SIZE_OF(statusOK));
}

int memoryUsed()
{
    initializeBioReceived();
    //find out how many fingerprint templates are used
    sendCommand(fingerprintCount, SIZE_OF(fingerprintCount));
    getReply();

#if DBG
    printf("memory used: %x", bioReceived[11]);
#endif
    
    //checkStatus will return 1 if status is OK, -1 if not ok
    return bioReceived[11]; //index that holds fingerprint id;
    
}

int matchFingerprint()
{
    int i=0;
    int id=-1;

    initializeBioReceived();
	
    //loop until a swipe was detected
    while (bioReceived[11]!=0x0A)
    {
	sendCommand(getImgCommand, SIZE_OF(getImgCommand));
	getReply();
    }
    
    //store image in char buffer
    sendCommand(storeImg2, SIZE_OF(storeImg2));
    getReply();

#if DBG
    printf("\nread data after stored image: \t");
    for (i=0; i<receivedIndex; i++)
    {	  
	printf("%x ", bioReceived[i]);
    }
    printf("\n");
#endif
    
    //search for a match
    sendCommand(search, SIZE_OF(search));
    getReply();
    
    id=verifyFingerprint();
    
#if DBG
     if (id==-1)
	 printf("\nNO MATCH FOUND\n");
     else
	 printf("\nMATCH FOUND ID: %d\t", id);
   
#endif
  
    return id;
}

void enrollCommand(u_int id)
{
    int i;
    u_int checksum=0x0;
    storeChar[12]=id-1; //index on biometric starts at 0

    initializeBioReceived();
	
    for (i=6; i<SIZE_OF(storeChar)-1; i++)
    {
	checksum=checksum+storeChar[i];
    }

    storeChar[SIZE_OF(storeChar)-1]=checksum;

#if DBG
    for(i=0; i<SIZE_OF(storeChar); i++)
    {
	printf("%x ", storeChar[i]);
    }
    printf("\n");
#endif
}

int enrollFingerprint(u_int id)
{
    int i=0;
    int count=0;
    int timeoutCount=20;
    
    enrollCommand(id);

    //loop until a swipe was detected
    while (bioReceived[11]!=0x0A && count < timeoutCount)
    {
	sendCommand(getImgCommand, SIZE_OF(getImgCommand));
	getReply();
	count++;
    }

    if (bioReceived[11]!=0x0A)
    {
	printf("timeout\n");
	return -1;
    }
    
    //store image in char buffer 1
    sendCommand(storeImg1, SIZE_OF(storeImg1));
    getReply();

    printf("second swipe\n");
    count=0;
    initializeBioReceived();

    //loop until a swipe was detected
    while (bioReceived[11]!=0x0A && count < timeoutCount)
    {
	
	sendCommand(getImgCommand, SIZE_OF(getImgCommand));
	getReply();
	count ++;
    }

    if (bioReceived[11]!=0x0A)
    {
	printf("timeout\n");
	return -1;
    }
    
    //store image in char buffer 2
    sendCommand(storeImg2, SIZE_OF(storeImg2));
    getReply();
    printf("stored second swipe\n");
#if DBG
    for(i=0; i<receivedIndex; i++)
    {
	printf("%x ", bioReceived[i]);
    }
    printf("\n");
#endif
    
    sendCommand(regModel, SIZE_OF(regModel));
    getReply();
    printf("registered\n");
#if DBG
    for(i=0; i<receivedIndex; i++)
    {
	printf("%x ", bioReceived[i]);
    }
    printf("\n");
#endif

    if (checkStatus(regFailed, SIZE_OF(regFailed)))
    {
	printf("REG FAILED\n");
	return 0;
    }
    else
    {
	sendCommand(storeChar, SIZE_OF(storeChar));
	getReply();

	if (checkStatus(statusOK, SIZE_OF(statusOK)))
	    printf("STORED!\n");
	else
	{
	    printf("ERROR!\n");
	    return 0;
	}
#if DBG
	for(i=0; i<receivedIndex; i++)
	{
	    printf("%x ", bioReceived[i]);
	}
	printf("\n");
#endif
    }
    
    return 1;
}

