#ifndef biometric_h
#define biometric_h

#include    "periph/stdc/std_c.h"

//return size of the array
#define SIZE_OF(x) (sizeof (x) / sizeof *(x))

//Command spec
#define PS_GetImage 		0x01
#define PS_GenChar 		0x02
#define PS_Match 		0x03
#define PS_Search		0x04
#define PS_RegModel		0x05
#define PS_StoreChar		0x06
#define PS_LoadChar		0x07
#define PS_UpChar		0x08
#define PS_DownChar		0x09
#define PS_DeleteChar		0x0C
#define PS_Empty		0x0D
#define PS_WriteReg		0x0E
#define PS_ReadSysPara		0x0F
#define PS_SetPws		0x12
#define PS_VfyPwd		0x13
#define PS_GetRandomCode	0x14
#define PS_SetChipAddr		0x15
#define PS_WriteNotepad		0x18
#define PS_ReadNotepad		0x19
#define PS_ValidTemplateNum	0x1D

//Pack structure
#define HEADER			0xEF01
#define	DEVICE_ADDR		0xFFFFFFFF

//Type of Packet
//Sending data packet
#define SEND_CMD_PACKET		0x01
#define SEND_DATA_PACKET	0x02
#define SEND_LAST_DATA_PACKET	0x08

//Command Response
#define RSP_CMD_PACKET		0x07
#define RSP_DATA_PACKET		0x02
#define RSP_LAST_DATA_PACKET	0x08

//Command packets
u_int deleteCommand[]={0xEF, 0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0x01, 0x00, 0x03, 0x0D, 0x00, 0x11};
u_int getImgCommand[]={0xEF, 0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0x01, 0x00, 0x03, 0x01, 0x00, 0x05};
u_int storeImg1[]={0xEF, 0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0x01, 0x00, 0x04, 0x02, 0x00, 0x00, 0x07};
u_int storeImg2[]={0xEF, 0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0x01, 0x00, 0x04, 0x02, 0x01, 0x00, 0x08};
u_int regModel[]={0xEF, 0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0x01, 0x00, 0x03, 0x05, 0x00, 0x09};
u_int storeChar[]={0xEF, 0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0x01, 0x00, 0x06, 0x06, 0x01, 0x00, 0x00, 0x00, 0x0E};
u_int search[]={0xEF, 0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0x01, 0x00, 0x08, 0x04, 0x01, 0x00, 0x00, 0x00, 0x79, 0x00, 0x87};
u_int fingerprintCount[]={0xEF, 0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0x01, 0x00, 0x03, 0x1D, 0x00, 0x21};
    
//Receive packets
u_int noMatch[]={0xEF, 0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0x07, 0x00, 0x07, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00, 0x17};
u_int bioReceived[]={0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
u_int regFailed[]={0xEF, 0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0x07, 0x00, 0x03, 0x0A, 0x00, 0x14};
u_int statusOK[]={0xEF, 0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0x07, 0x00, 0x03, 0x00, 0x00, 0x0A};

//functions
int verifyFingerprint();
int checkStatus(u_int status[], int size);
void sendCommand(u_int command[], int size);
void initializeBioReceived();
void getReply();
int deleteAll();
int memoryUsed();
int matchFingerprint();
void enrollCommand(u_int id);
int enrollFingerprint(u_int id);

#endif
