# include "smartAKK.h"

/* Master Clock Frequency Detection. Output: Frequency in kHz */
u_int mcki_detect ( void )
{
    int period;

    at91_capture_open ( &TC0_DESC, MODE_TIOA_PERIOD ) ;
    while ( at91_capture_get_status ( &TC0_DESC ) != TRUE ) ;
    period = at91_capture_read ( &TC0_DESC ) ;
    at91_capture_close ( &TC0_DESC ) ;

    if ( period > 15 )
        return ( MCKKHz ) ;
    if ( period > 7 )
        return ( MCKKHz >> 1 ) ;
    if ( period > 3 )
        return ( MCKKHz >> 2 ) ;
    if ( period > 1 )
        return ( MCKKHz >> 3 ) ;

    return ( MCKKHz >> 4 ) ;
}

void validateAction(int validLED)
{
    switch(validLED)
    {
	case 1:
	    at91_pio_write (&PIOB_DESC, LED1, LED_ON );
	    break;
	case 2:
	    at91_pio_write (&PIOB_DESC, LED2, LED_ON ) ;
	    break;
	case 3:
	    at91_pio_write (&PIOB_DESC, LED3, LED_ON ) ;
	    break;
	case 4:
	    at91_pio_write (&PIOB_DESC, LED4, LED_ON ) ;
	    break;
	case 5:
	    at91_pio_write (&PIOB_DESC, LED5, LED_ON ) ;
	    break;
	case 6:
	    at91_pio_write (&PIOB_DESC, LED6, LED_ON ) ;
	    break;
	default:
	    at91_pio_write (&PIOB_DESC, LED8, LED_ON ) ;
	    break;
    }

}

void invalidAction(int invalidLED)
{
    int i;

    switch(invalidLED)
    {
	case 1:
	    for(i = 0; i < 6; i++)
	    {
		at91_pio_write (&PIOB_DESC, LED1, LED_ON );
		wait_desc.period = 200000 ;
		at91_wait_open ( &wait_desc ) ;
		at91_pio_write (&PIOB_DESC, LED1, LED_OFF );
		at91_wait_open ( &wait_desc ) ;
	    }
	    break;
	case 2:
	    for(i = 0; i < 6; i++)
	    {

		at91_pio_write (&PIOB_DESC, LED2, LED_ON );
		wait_desc.period = 200000 ;
		at91_wait_open ( &wait_desc ) ;
		at91_pio_write (&PIOB_DESC, LED2, LED_OFF );
		at91_wait_open ( &wait_desc ) ;
	    }
	    break;
	case 3:
	    for(i = 0; i < 6; i++)
	    {
		at91_pio_write (&PIOB_DESC, LED3, LED_ON );
		wait_desc.period = 200000 ;
		at91_wait_open ( &wait_desc ) ;
		at91_pio_write (&PIOB_DESC, LED3, LED_OFF );
		at91_wait_open ( &wait_desc ) ;
	    }
	    break;
	case 4:
	    for(i = 0; i < 6; i++)
	    {
		at91_pio_write (&PIOB_DESC, LED4, LED_ON );
		wait_desc.period = 200000 ;
		at91_wait_open ( &wait_desc ) ;
		at91_pio_write (&PIOB_DESC, LED4, LED_OFF );
		at91_wait_open ( &wait_desc ) ;
	    }
	    break;
	case 5:
	    for(i = 0; i < 6; i++)
	    {
		at91_pio_write (&PIOB_DESC, LED5, LED_ON );
		wait_desc.period = 200000 ;
		at91_wait_open ( &wait_desc ) ;
		at91_pio_write (&PIOB_DESC, LED5, LED_OFF );
		at91_wait_open ( &wait_desc ) ;
	    }
	    break;
	case 6:
	    for(i = 0; i < 6; i++)
	    {
		at91_pio_write (&PIOB_DESC, LED6, LED_ON );
		wait_desc.period = 200000 ;
		at91_wait_open ( &wait_desc ) ;
		at91_pio_write (&PIOB_DESC, LED6, LED_OFF );
		at91_wait_open ( &wait_desc ) ;
	    }
	    break;
	default:
	    for(i = 0; i < 8; i++)
	    {
		at91_pio_write (&PIOB_DESC, LED8, LED_ON );
		wait_desc.period = 200000 ;
		at91_wait_open ( &wait_desc ) ;
		at91_pio_write (&PIOB_DESC, LED8, LED_OFF );
		at91_wait_open ( &wait_desc ) ;
	    }
	    break;
    }
}

void initAll(void)
{
    buttonV0 = 0x0;
    buttonV1 = 0x0;

    encV0 = 0x0; 
    encV1 = 0x0;

        /* AH: LED Initialization */
    at91_pio_open (&PIOB_DESC, LED_MASK, PIO_OUTPUT ) ;
    at91_pio_write (&PIOB_DESC, LED_MASK, LED_OFF ) ;
}

int main( void )
{
    int i = 0;
    int id = 0;
    u_int data_temp = 0x0;
    u_int data_send = 0xFF;

    unsigned int serialNum = 0x0;

    initAll();

    /* AH: Master clock detection */
    wait_desc.mcki_khz = 32000 ;

    /* AH: Initialize and open USARTS */
    at91_usart_open(&USART1_DESC, (u_int) US_ASYNC_MODE, (u_int) BAUDS19200, 0);
    at91_usart_open(&USART2_DESC, (u_int) US_ASYNC_MODE, (u_int) BAUDS38400, 0);

    while(1)
    {
	initAll();
	validateAction(systemReady);
	//always get fingerprint first
	id = matchFingerprint();

	if (id > 0)
	{
	    validateAction(fingerPrint);
	    printf("we are back from validate fingerprint\n");

	    buttonV0 = getAKVcommand(id, wait_desc);
	    //Check if we got a timeout or not here

	    /* increment serial number count */
	    serialNum = serialNum + SERIAL_COUNT;
	    buttonV1 = serialNum;

	    if(buttonV0 != 0x0)
		validateAction(buttonSequence);
	    else
	    {
		invalidAction(buttonSequence);
		continue;
	    }

	    printf("we  have gotten a button and it was %x\n", buttonV0);
	    
	    if (buttonV0 == 0x55555555)
	    {
		validateAction(enrollNewFinger);
		wait_desc.period = 300000 ;
		at91_wait_open ( &wait_desc ) ;
		continue;
	    }

	    if (buttonV0 == 0x66666666)
	    {
		validateAction(enrollNewMaster);
		wait_desc.period = 300000 ;
		at91_wait_open ( &wait_desc ) ;
		continue;
	    }
# if ADBG
	    printf("AKV ACTION\t\t[%X %X]\t", buttonV0, buttonV1);
# endif
	    encV0 = encipherA(buttonV0, buttonV1, VIN0, VIN1, VIN2, VIN3);
	    encV1 = encipherB(buttonV0, buttonV1, VIN0, VIN1, VIN2, VIN3);

	    if(encV0 != 0x0)
		validateAction(encryption);
	    else
	    {
		invalidAction(buttonSequence);
		wait_desc.period = 300000 ;
		at91_wait_open ( &wait_desc ) ;
		continue;
	    }    
# if ADBG
	    printf("ENCRYPTED\t[%X %X]\n", encV0, encV1);
# endif

/* AH: Signal start of packet transmission */
	    if(sendByte(startPacketByte) != 1)
		invalidAction(packet);

	    /* AH: AKK will send a 4 byte packet */
	    for (i = 0; i < 4; i++)
	    {
		data_temp = encV0;
		data_temp = data_temp >> 8*(4-i-1);

	    /* XBee sends 1 bytes (8 bits) at a time */
		if(sendByte(data_temp&data_send) != 1)
		    invalidAction(packet);
	    }

	    /* AH: AKK will send a 4 byte packet */
	    for (i = 0; i < 4; i++)
	    {
		data_temp = encV1;
		data_temp = data_temp >> 8*(4-i-1);

		/* XBee sends 1 bytes (8 bits) at a time */
		if(sendByte(data_temp&data_send) != 1)
		    invalidAction(packet);
	    }

	    /* AH: Signal end of packet transmission */
	    if(sendByte(endPacketByte) == 1)
		validateAction(packet);
	    else
		invalidAction(packet);


	    wait_desc.period = 300000 ;
	    at91_wait_open ( &wait_desc ) ;
	  
	}//if id
	else
	    invalidAction(fingerPrint);

    }/*while*/




    /* Close the USART */
    at91_usart_close(&USART2_DESC);
    return(0);
}


